/******************************************************************************

    Program:        PDFtool.p

    Written By:     Gordon Campbell - PRO-SYS Consultants Ltd.
    Written On:     January 6, 2004

    Description:    This program will contain a bunch of tools for helping to
                    build PDF documents.

                    Tools currently available are:

                    - TABLE (see BuildTable).


    Note:           This code was separated into this procedure to ensure
                    that we don't cause any memory or compilation issues with
                    the current PDFinclude procedure.

                    Also, when running PDFinclude procedures you need to run
                    them with reference to pCallProc. For example:

                    RUN pdf_text IN pCallProc (pdfStream, "Text", 10).


******************************************************************************/

DEFINE INPUT PARAMETER pCallProc  AS HANDLE NO-UNDO.

{ pdf_func.i pCallProc }

DEFINE VARIABLE h_DataQuery   AS HANDLE NO-UNDO.
DEFINE VARIABLE h_DataBuffer  AS HANDLE NO-UNDO.
DEFINE VARIABLE h_DataField   AS HANDLE NO-UNDO.

DEFINE VARIABLE c_OldDetailFont     AS CHARACTER NO-UNDO.
DEFINE VARIABLE c_DetailFont        AS CHARACTER NO-UNDO.

DEFINE VARIABLE d_DetailFontSize    AS DECIMAL NO-UNDO.
DEFINE VARIABLE d_OldDetailFontSize AS DECIMAL NO-UNDO.


PROCEDURE BuildTable:
  DEFINE INPUT PARAMETER pdfStream   AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pdfToolName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pDataBuffer AS HANDLE NO-UNDO.

  DEFINE VARIABLE c_Font        AS CHARACTER NO-UNDO.

  DEFINE VARIABLE d_FontSize    AS DECIMAL NO-UNDO.

  /* Store old values */
  c_OldDetailFont = pdf_Font(pdfStream).
  d_OldDetailFontSize = pdf_PointSize(pdfStream).

  /* Determine Font and Font Size for Header Labels */
  c_DetailFont = pdf_get_tool_parameter (pdfStream,
                                   pdfToolname,
                                   "DetailFont",
                                   0).
  d_DetailFontSize = DEC(pdf_get_tool_parameter (pdfStream,
                                                 pdfToolname,
                                                 "DetailFontSize",
                                                 0)) NO-ERROR.

  /* Create QUery and start processing data */
  CREATE QUERY h_DataQuery.

  h_DataBuffer = pDataBuffer:DEFAULT-BUFFER-HANDLE.

  h_DataQuery:ADD-BUFFER(h_DataBuffer:NAME).
  h_DataQuery:QUERY-PREPARE("PRESELECT EACH " + h_DataBuffer:NAME + " NO-LOCK").
  h_DataQuery:QUERY-OPEN.

  h_DataQuery:GET-FIRST(NO-LOCK).

  /* Handle Table Header */
  IF h_DataBuffer:AVAILABLE THEN DO:

    /* Perform some Table Preprocessing.  For example, we need to determine the
       actual Y-axis starting points for each Column -- based on the Column Header
       Width */
    RUN PreProcessTable(pdfStream, pdfToolName).

    RUN DisplayTableHeader (pdfStream, pdfToolName).

    REPEAT WHILE h_DataBuffer:AVAIL:

      RUN DisplayTableDetail(pdfStream,pdfToolName).

      h_DataQuery:GET-NEXT(NO-LOCK).
    END.
  END. /* Data Available */

  /* Reset old values */
  RUN pdf_set_font IN pCallProc (pdfStream, c_OldDetailFont, d_OldDetailFontSize).

  DELETE WIDGET h_DataQuery.

END. /* BuildTable */

PROCEDURE PreProcessTable:
  DEFINE INPUT PARAMETER pdfStream    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pdfToolName  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE c_ColLabel    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_ColWidth    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_Font        AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_ColX        AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_ColPad      AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_OldFont     AS CHARACTER NO-UNDO.

  DEFINE VARIABLE d_FontSize    AS DECIMAL NO-UNDO.
  DEFINE VARIABLE d_OldFontSize AS DECIMAL NO-UNDO.

  DEFINE VARIABLE i_Field       AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_ColWidth    AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_ColPoints   AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_ColX        AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_ColPad      AS INTEGER NO-UNDO.

  DEFINE VARIABLE i_RunX        AS INTEGER NO-UNDO.

  i_RunX = pdf_LeftMargin(pdfStream).
  IF i_RunX = 0 THEN i_RunX = 1.

  /* Store old values */
  c_OldFont = pdf_Font(pdfStream).
  d_OldFontSize = pdf_PointSize(pdfStream).

  /* Determine if the Font and Font Size have been setup for the Header.
     If not, then set them to the current Font and Font Size */
  c_Font = pdf_get_tool_parameter (pdfStream,
                                   pdfToolname,
                                   "HeaderFont",
                                   0).
  IF c_Font = "" THEN
    RUN pdf_set_tool_parameter IN pCallProc
        (pdfstream,
         pdfToolName,
         "HeaderFont",
         0,
         pdf_Font(pdfStream)).

  d_FontSize = DEC(pdf_get_tool_parameter (pdfStream,
                                           pdfToolname,
                                           "HeaderFontSize",
                                           0)) NO-ERROR.
  IF d_FontSize = 0 THEN
    RUN pdf_set_tool_parameter IN pCallProc
        (pdfstream,
         pdfToolName,
         "HeaderFontSize",
         0,
         STRING(pdf_PointSize(pdfStream))).

  /* Set the Font to the Header Size so that we can ensure that we don't
     overrun our columns */
  RUN pdf_set_Font IN pCallProc
                     (pdfStream, c_Font, d_FontSize).

  /* Determine if the Column Padding is setup -- if not default to 5 points.
    This should be setup as a default when adding the Table */
  i_ColPad = INT(pdf_get_tool_parameter (pdfStream,
                                   pdfToolname,
                                   "ColumnPadding",
                                   0)) NO-ERROR.
  IF i_ColPad = 0 THEN DO:
    RUN pdf_set_tool_parameter IN pCallProc
        (pdfstream,
         pdfToolName,
         "ColumnPadding",
         0,
         "5").
    i_ColPad = 5.
  END.

  /* Now PreProcess for each Column */
  DO i_Field = 1 TO h_DataBuffer:NUM-FIELDS:
    h_DataField = h_DataBuffer:BUFFER-FIELD(i_Field).

    /* Determine Column Label */
    c_ColLabel = pdf_get_tool_parameter(pdfStream,
                                        pdfToolname,
                                        "ColumnHeader",
                                        i_Field).

    /* Determine Column Width - initially in Characters but convert to
       Points */
    c_ColWidth = pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         "ColumnWidth",
                                         i_Field).

    i_ColWidth = INT(c_ColWidth) NO-ERROR.
    IF i_ColWidth = 0 THEN DO:

      i_ColWidth = LENGTH(STRING('',h_DataField:FORMAT), "character":U).
      IF LENGTH(c_ColLabel, "character":U) > i_ColWidth THEN
        i_ColWidth = LENGTH(c_ColLabel, "character":U).

      RUN pdf_set_tool_parameter IN pCallProc
          (pdfStream,pdfToolName,"ColumnWidth",i_Field,STRING(i_ColWidth)).
    END.

    /* Determine Column Placement - in Points -- if set */
    c_ColX = pdf_get_tool_parameter (pdfStream,
                                     pdfToolname,
                                     "ColumnX",
                                     i_Field).
    i_ColX = INT(c_ColX) NO-ERROR.

    /* If zero then the Y Point hasn't been set so we need to determine the
       default Columns - in points - and we will use the Header font, font size,
       and ColumnWidth parameters to determine the value */
    IF i_ColX = 0 THEN DO:

      /* Determine Font and Font Size for Header Labels -- assuming that the
         Header Label will be at minimum the same size as the detail text. Add:

         Current X Placement
         Column Padding
         Header Width - based on selected Header Font and Font Size
      */
      i_ColX = i_RunX + i_ColPad .

      RUN pdf_set_tool_parameter IN pCallProc
          (pdfStream,
           pdfToolName,
           "ColumnX",
           i_Field,
           STRING(i_ColX)).

      RUN pdf_set_tool_parameter IN pCallProc
          (pdfStream,
           pdfToolName,
           "MaxX",
           i_Field,
           STRING(i_ColPad + pdf_text_widthdec(pdfStream,FILL("D",i_ColWidth)))).

      i_RunX = i_RunX + i_ColPad + pdf_text_widthdec(pdfStream,FILL("D",i_ColWidth)).
    END.

  END. /* Num Fields */

  /* Reset old values */
  RUN pdf_set_font IN pCallProc (pdfStream, c_OldFont, d_OldFontSize).

END. /* PreProcessTable */

PROCEDURE DisplayTableHeader:
  DEFINE INPUT PARAMETER pdfStream    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pdfToolName  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE c_ColLabel    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_ColWidth    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_Font        AS CHARACTER NO-UNDO.
  DEFINE VARIABLE c_OldFont     AS CHARACTER NO-UNDO.

  DEFINE VARIABLE d_FontSize      AS DECIMAL NO-UNDO.
  DEFINE VARIABLE d_OldFontSize   AS DECIMAL NO-UNDO.
  DEFINE VARIABLE d_Outline       AS DECIMAL NO-UNDO.
  DEFINE VARIABLE d_BGRed         AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_BGGreen       AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_BGBlue        AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_HdrTextRed    AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_HdrTextGreen  AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_HdrTextBlue   AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_DtlTextRed    AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_DtlTextGreen  AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_DtlTextBlue   AS DECIMAL DECIMALS 4 NO-UNDO.

  DEFINE VARIABLE i_ColWidth    AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_Field       AS INTEGER NO-UNDO.

  DEFINE VARIABLE i_RunWidth    AS INTEGER NO-UNDO INIT 1.

  /* Store old values */
  c_OldFont = pdf_Font(pdfStream).
  d_OldFontSize = pdf_PointSize(pdfStream).

  /* Determine Font and Font Size for Header Labels */
  c_Font = pdf_get_tool_parameter (pdfStream,
                                   pdfToolname,
                                   "HeaderFont",
                                   0).
  d_FontSize = DEC(pdf_get_tool_parameter (pdfStream,
                                           pdfToolname,
                                           "HeaderFontSize",
                                           0)) NO-ERROR.

  RUN pdf_set_Font IN pCallProc
                     (pdfStream, c_Font, d_FontSize).

  /* Determine the Outline WIdth - if used */
  d_Outline = DEC(pdf_get_tool_parameter (pdfStream,
                                          pdfToolname,
                                          "Outline",
                                          0)) NO-ERROR.

  RUN DetermineColor (INPUT  pdfStream,
                      INPUT  pdfToolname,
                      INPUT  "HeaderBGColor",
                      INPUT  0,
                      OUTPUT d_BGred,
                      OUTPUT d_BGgreen,
                      OUTPUT d_BGblue).

  RUN DetermineColor (INPUT  pdfStream,
                      INPUT  pdfToolname,
                      INPUT  "HeaderTextColor",
                      INPUT  0,
                      OUTPUT d_HdrTextred,
                      OUTPUT d_HdrTextgreen,
                      OUTPUT d_HdrTextblue).

  RUN DetermineColor (INPUT  pdfStream,
                      INPUT  pdfToolname,
                      INPUT  "DetailTextColor",
                      INPUT  0,
                      OUTPUT d_DtlTextred,
                      OUTPUT d_DtlTextgreen,
                      OUTPUT d_DtlTextblue).

  RUN pdf_text_color in pCallProc (pdfStream,d_HdrTextred,d_HdrTextgreen,d_HdrTextblue).

  /* Now produce the Column Headers */
  DO i_Field = 1 TO h_DataBuffer:NUM-FIELDS:
    h_DataField = h_DataBuffer:BUFFER-FIELD(i_Field).

    /* Determine Column Label */
    c_ColLabel = pdf_get_tool_parameter(pdfStream,
                                        pdfToolname,
                                        "ColumnHeader",
                                        i_Field).

    /* Determine Column Width */
    c_ColWidth = pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         "ColumnWidth",
                                         i_Field).
    i_ColWidth = INT(c_ColWidth) NO-ERROR.
    IF i_ColWidth = 0 THEN DO:

      i_ColWidth = LENGTH(STRING('',h_DataField:FORMAT), "character":U).
      IF LENGTH(c_ColLabel, "character":U) > i_ColWidth THEN
        i_ColWidth = LENGTH(c_ColLabel, "character":U).

      RUN pdf_set_tool_parameter IN pCallProc
          (pdfStream,pdfToolName,"ColumnWidth",i_Field,STRING(i_ColWidth)).
    END.

    RUN pdf_text_xy IN pCallProc
        (pdfStream, c_ColLabel,
                    pdf_get_tool_parameter (pdfStream,
                                            pdfToolname,
                                            "ColumnX",
                                            i_Field),
                    pdf_textY(pdfStream)).

    /* If Outline is not zero then draw a Rectangle around each of the
       Header Columns */
    IF d_Outline <> 0 THEN DO:
      RUN pdf_stroke_fill in pCallProc (pdfStream,d_BGred,d_BGgreen,d_BGblue).

      RUN pdf_rect IN pCallProc
          (pdfStream,
           INT( pdf_get_tool_parameter (pdfStream,
                                  pdfToolname,
                                  "ColumnX",
                                  i_Field) )
           -
           INT( pdf_get_tool_parameter (pdfStream,
                                  pdfToolname,
                                  "ColumnPadding",
                                  0) ),
           pdf_textY(pdfStream) - 2,
           pdf_get_tool_parameter (pdfStream,
                                  pdfToolname,
                                  "MaxX",
                                  i_Field),
           pdf_PointSize(pdfStream) + 2,
           d_Outline).

    END. /* Outline */

  END. /* Num Fields */

  /* Set to Detail Values */
  RUN pdf_set_font IN pCallProc (pdfStream, c_OldFont, d_OldFontSize).
  RUN pdf_set_Font IN pCallProc
                     (pdfStream, c_DetailFont, d_DetailFontSize).
  RUN pdf_text_color in pCallProc (pdfStream,d_DtlTextred,d_DtlTextgreen,d_DtlTextblue).

  /* Skip to the next line */
  RUN pdf_skip IN pCallProc
      (pdfStream).

  RUN pdf_set_TextY IN pCallProc
                   (pdfStream,
                    pdf_TextY(pdfStream)
                    - INT(pdf_get_tool_parameter (pdfStream,
                                                  pdfToolname,
                                                  "ColumnPadding",
                                                  0)) * 2).

  RUN pdf_set_tool_parameter IN pCallProc
      (pdfStream,pdfToolName,"MaxY",0,STRING(pdf_textY(pdfStream))).

END. /* DisplayTableHeader*/

PROCEDURE DisplayTableDetail:
  DEFINE INPUT PARAMETER pdfStream    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pdfToolName  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE i_Field       AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_ColWidth    AS INTEGER NO-UNDO.

  /* Display Table Detail */
  DO i_Field = 1 TO h_DataBuffer:NUM-FIELDS:
    h_DataField = h_DataBuffer:BUFFER-FIELD(i_Field).

    i_ColWidth = INT(pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         "ColumnWidth",
                                         i_Field)).

    RUN pdf_text_xy IN pCallProc
        (pdfStream, SUBSTR(h_DataField:BUFFER-VALUE(),1,i_ColWidth, "character":U),
                    pdf_get_tool_parameter (pdfStream,
                                            pdfToolname,
                                            "ColumnX",
                                            i_Field),
                    pdf_textY(pdfStream)).

  END.

  RUN pdf_skip IN pCallProc
      (pdfStream).

END. /* DisplayTableDetail */

PROCEDURE DisplayTableOutline:
  DEFINE INPUT PARAMETER pdfStream    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pdfToolName  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE d_Outline     AS DECIMAL NO-UNDO.
  DEFINE VARIABLE d_BGred       AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_BGgreen     AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE VARIABLE d_BGblue      AS DECIMAL DECIMALS 4 NO-UNDO.

  DEFINE VARIABLE i_Field       AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_Height      AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_X           AS INTEGER NO-UNDO.

  /* Determine the Outline WIdth - if used */
  d_Outline = DEC(pdf_get_tool_parameter (pdfStream,
                                          pdfToolname,
                                          "Outline",
                                          0)) NO-ERROR.

  IF d_Outline > 0 THEN DO:
    RUN DetermineColor (INPUT  pdfStream,
                        INPUT  pdfToolname,
                        INPUT  "DetailBGColor",
                        INPUT  0,
                        OUTPUT d_BGred,
                        OUTPUT d_BGgreen,
                        OUTPUT d_BGblue).

    RUN pdf_stroke_fill IN pCallProc (pdfStream,d_BGred,d_BGgreen,d_BGblue).

    /* Determine the outline height - for the given page */
    i_Height = INT(pdf_get_tool_parameter (pdfStream,
                                           pdfToolname,
                                           "MaxY",
                                           0))
             - pdf_TextY(pdfstream)
             + INT(pdf_get_tool_parameter (pdfStream,
                                           pdfToolname,
                                           "ColumnPadding",
                                           0))
             + INT(pdf_get_tool_parameter (pdfStream,
                                           pdfToolname,
                                           "DetailFontSize",
                                           0)).

    /* Display Table Detail */
    DO i_Field = 1 TO h_DataBuffer:NUM-FIELDS:
      h_DataField = h_DataBuffer:BUFFER-FIELD(i_Field).

      i_X = INT( pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         "ColumnX",
                                         i_Field) )
          - INT( pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         "ColumnPadding",
                                         0) ).

      RUN pdf_rect IN pCallProc
          (pdfStream,
           i_X,
           pdf_textY(pdfStream) - 2,
           pdf_get_tool_parameter (pdfStream,
                                  pdfToolname,
                                  "MaxX",
                                  i_Field),
           i_Height,
           d_Outline).

    END. /* Fields */

  END. /* Outline > 0 */

END. /* DisplayTableOutline */

PROCEDURE DetermineColor:
  DEFINE INPUT  PARAMETER pdfStream    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pdfToolName  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pdfToolParam AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pdfToolCol   AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER pdfRed       AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE OUTPUT PARAMETER pdfGreen     AS DECIMAL DECIMALS 4 NO-UNDO.
  DEFINE OUTPUT PARAMETER pdfBlue      AS DECIMAL DECIMALS 4 NO-UNDO.

  DEFINE VARIABLE i_BGRed       AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_BGGreen     AS INTEGER NO-UNDO.
  DEFINE VARIABLE i_BGBlue      AS INTEGER NO-UNDO.

  DEFINE VARIABLE c_ParamValue  AS CHARACTER NO-UNDO.

  c_ParamValue = pdf_get_tool_parameter (pdfStream,
                                         pdfToolname,
                                         pdfToolParam,
                                         pdfToolCol).

  /* If not found the default to BLACK */
  IF c_ParamValue = "" THEN
    RETURN.

  pdfRed   = INT(ENTRY(1, c_ParamValue)) / 255.
  pdfGreen = INT(ENTRY(2, c_ParamValue)) / 255.
  pdfBlue  = INT(ENTRY(3, c_ParamValue)) / 255.

END.

/* end of PDFtool.p */
